/*
 * app.c
 *
 * Main event loop etc. for the Red Feline Backup application
 *
 * Copyright (C) 2004 Daniel Thompson <d.thompson@gmx.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include "rfbackup.h"

static void AppStart(void)
{
	/* TODO: manage preferences if required */
	
	FrmGotoForm(kFormMain);
}

static void AppStop(void)
{
	/* TODO: manage preferences if required */
}

static Boolean AppHandleEvent(EventPtr event)
{
	Boolean eventHandled = true;
	Int formID;
	FormPtr form;
	UInt16 buttonID;

	switch (event->eType) {
	case frmLoadEvent:
		formID = event->data.frmLoad.formID;
		form = FrmInitForm(formID);
		
		FrmSetActiveForm(form);

		/* choose a form handler based on the form ID */
		switch (formID) {
		case kFormMain:
			FrmSetEventHandler(form, MainFormHandleEvent);
			break;
		case kFormProgress:
			FrmSetEventHandler(form, ProgressFormHandleEvent);
			break;
		default:
			ErrFatalDisplay("Invalid Form Load Event");
			break;
		}
		
		break;
		
	case menuEvent:
		/* handle menu events that have the same action for all forms */
		switch (event->data.menu.itemID) {
		case kMenuItemAbout:
			form = FrmInitForm(kFormAbout);
			buttonID = FrmDoDialog(form);
			FrmDeleteForm(form);
			break;
			
#ifdef DEBUG_BUILD
		case kMenuItemSoftReset:
			if (0 == FrmCustomAlert(kAlertGenericConfirmation,
			                        "Really reset this device?", "", "")) {
				SysReset();
			}
			break;
			
		case kMenuItemColdReset:
			if (0 == FrmCustomAlert(kAlertGenericConfirmation,
						"Really delete all data on this device and reset?", "", "")) {
				SysColdBoot(0, 0, 0, 0, 0);
			
			}
			break;	
#endif
				 
		default:
			eventHandled = false;
			break;
		}
		break;

	default:
		eventHandled = false;
		break;
	}

	return eventHandled;
}

Boolean AppHandleEventsUntil(eventsEnum eType)
{
	EventType event;
	UInt16 err;
	Boolean eventHandled;
	
	/* this is our main event loop */
	do {
		EvtGetEvent(&event, evtWaitForever);

		/* ANSI C mandates lazy evaluation! */
		eventHandled = SysHandleEvent(&event) ||
		               MenuHandleEvent(0, &event, &err) ||
		               AppHandleEvent(&event) ||
		               FrmDispatchEvent(&event);
		               
	} while (eType != event.eType && appStopEvent != event.eType);
	
	if (eType == event.eType) {
		return true;
	}
	
	/* this is an appStopEvent during an inner event loop, we requeue and drop upwards */
	EvtAddEventToQueue(&event);
	return false;
}

DWord PilotMain (UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		AppStart();
		AppHandleEventsUntil(appStopEvent);
		AppStop();
		FrmCloseAllForms();
	}

	return 0;
}