/*
 * progressform.c
 *
 * Manage the backup progress form
 *
 * Copyright (C) 2004 Daniel Thompson <d.thompson@gmx.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
 
#include "rfbackup.h"

static void FrmSetLabel(FormType *form, UInt16 labelID, const Char *string)
{
	UInt16 labelObjectIndex = FrmGetObjectIndex(form, labelID);
	
	FrmHideObject(form, labelObjectIndex);
	FrmCopyLabel(form, labelID, string);
	FrmShowObject(form, labelObjectIndex);
}

static void ProgressFormUpdateDatabaseLabel(const Char *dbName)
{
	FrmSetLabel(FrmGetActiveForm(), kLabelDatabase, dbName);
}

void ProgressFormDoDialog(UInt16 mode)
{
	Err err = dmErrCorruptDatabase;
	FormPtr form;
	UInt16 volRefNum;
	UInt32 volIterator = vfsIteratorStart;
	Boolean skipReset = false;

	/* assert our preconditions */
	ErrNonFatalDisplayIf(mode != progressFormBackupToVFS && mode != progressFormRestoreFromVFS,
	                     "Invalid backup mode");

	/* bring up the progress form */
	FrmPopupForm(kFormProgress);
	if (!AppHandleEventsUntil(frmOpenEvent)) {
		/* application is shutting down */
		return;
	}
	
	/* update the form */
	form = FrmGetActiveForm();
	/* TODO: set title appropraitely */

	err = VFSVolumeEnumerate(&volRefNum, &volIterator);
	if (errNone == err) {
		err = (mode == progressFormBackupToVFS    ? BakBackupToVFS : BakRestoreFromVFS) (
				volRefNum, "/PALM/RFBackup", ProgressFormUpdateDatabaseLabel);
	}
	
	switch (err) {
	case expErrEnumerationEmpty:
		(void) FrmAlert(kAlertNoMemoryCard);
		skipReset = true;
		break;
		
	case errNone:
		break;
	
	default:
		ErrAlert(err);
	}

	if (progressFormRestoreFromVFS == mode && !skipReset) {
		(void) FrmAlert(kAlertReset);
		SysReset();
	}

	FrmReturnToForm(0);
}

Boolean ProgressFormHandleEvent(EventPtr event)
{
	Boolean eventHandled = false;
	FormPtr form;
	
	form = FrmGetActiveForm();
	
	switch (event->eType) {
	case frmOpenEvent:
		FrmSetLabel(form, kLabelDatabase, "");
		/*FALLTHRU*/
	case frmUpdateEvent:
		FrmDrawForm(form);
		eventHandled = true;
		break;

	default:
		break;
	}

	return eventHandled;
}
 